@@ -1,23 +1,23 @@
 #ifndef NVIM_EVAL_TYPVAL_H
 #define NVIM_EVAL_TYPVAL_H
 
+#include <assert.h>
 #include <inttypes.h>
+#include <stdbool.h>
 #include <stddef.h>
 #include <string.h>
-#include <stdbool.h>
-#include <assert.h>
 
-#include "nvim/types.h"
-#include "nvim/hashtab.h"
-#include "nvim/garray.h"
-#include "nvim/mbyte.h"
 #include "nvim/func_attr.h"
-#include "nvim/lib/queue.h"
-#include "nvim/profile.h"  // for proftime_T
-#include "nvim/pos.h"      // for linenr_T
+#include "nvim/garray.h"
 #include "nvim/gettext.h"
-#include "nvim/message.h"
+#include "nvim/hashtab.h"
+#include "nvim/lib/queue.h"
 #include "nvim/macros.h"
+#include "nvim/mbyte.h"
+#include "nvim/message.h"
+#include "nvim/pos.h"      // for linenr_T
+#include "nvim/profile.h"  // for proftime_T
+#include "nvim/types.h"
 #ifdef LOG_LIST_ACTIONS
 # include "nvim/memory.h"
 #endif
@@ -30,7 +30,7 @@ typedef uint64_t uvarnumber_T;
 typedef double float_T;
 
 /// Refcount for dict or list that should not be freed
-enum { DO_NOT_FREE_CNT = (INT_MAX / 2) };
+enum { DO_NOT_FREE_CNT = (INT_MAX / 2), };
 
 /// Additional values for tv_list_alloc() len argument
 enum ListLenSpecials {
@@ -55,7 +55,7 @@ enum ListLenSpecials {
 #define VARNUMBER_MAX INT64_MAX
 #define UVARNUMBER_MAX UINT64_MAX
 
-/// Mimimal possible value of varnumber_T variable
+/// Minimal possible value of varnumber_T variable
 #define VARNUMBER_MIN INT64_MIN
 
 /// %d printf format specifier for varnumber_T
@@ -64,6 +64,7 @@ enum ListLenSpecials {
 typedef struct listvar_S list_T;
 typedef struct dictvar_S dict_T;
 typedef struct partial_S partial_T;
+typedef struct blobvar_S blob_T;
 
 typedef struct ufunc ufunc_T;
 
@@ -71,16 +72,20 @@ typedef enum {
   kCallbackNone = 0,
   kCallbackFuncref,
   kCallbackPartial,
+  kCallbackLua,
 } CallbackType;
 
 typedef struct {
   union {
     char_u *funcref;
     partial_T *partial;
+    LuaRef luaref;
   } data;
   CallbackType type;
 } Callback;
-#define CALLBACK_NONE ((Callback){ .type = kCallbackNone })
+
+#define CALLBACK_INIT { .type = kCallbackNone }
+#define CALLBACK_NONE ((Callback)CALLBACK_INIT)
 
 /// Structure holding dictionary watcher
 typedef struct dict_watcher {
@@ -89,12 +94,17 @@ typedef struct dict_watcher {
   size_t key_pattern_len;
   QUEUE node;
   bool busy;  // prevent recursion if the dict is changed in the callback
+  bool needs_free;
 } DictWatcher;
 
+/// Bool variable values
+typedef enum {
+  kBoolVarFalse,         ///< v:false
+  kBoolVarTrue,          ///< v:true
+} BoolVarValue;
+
 /// Special variable values
 typedef enum {
-  kSpecialVarFalse,  ///< v:false
-  kSpecialVarTrue,   ///< v:true
   kSpecialVarNull,   ///< v:null
 } SpecialVarValue;
 
@@ -114,9 +124,11 @@ typedef enum {
   VAR_LIST,         ///< List, .v_list is used.
   VAR_DICT,         ///< Dictionary, .v_dict is used.
   VAR_FLOAT,        ///< Floating-point value, .v_float is used.
-  VAR_SPECIAL,      ///< Special value (true, false, null), .v_special
+  VAR_BOOL,         ///< true, false
+  VAR_SPECIAL,      ///< Special value (null), .v_special
                     ///< is used.
   VAR_PARTIAL,      ///< Partial, .v_partial is used.
+  VAR_BLOB,         ///< Blob, .v_blob is used.
 } VarType;
 
 /// Structure that holds an internal variable value
@@ -125,13 +137,15 @@ typedef struct {
   VarLockStatus v_lock;  ///< Variable lock status.
   union typval_vval_union {
     varnumber_T v_number;  ///< Number, for VAR_NUMBER.
+    BoolVarValue v_bool;        ///< Bool value, for VAR_BOOL
     SpecialVarValue v_special;  ///< Special value, for VAR_SPECIAL.
     float_T v_float;  ///< Floating-point number, for VAR_FLOAT.
     char_u *v_string;  ///< String, for VAR_STRING and VAR_FUNC, can be NULL.
     list_T *v_list;  ///< List for VAR_LIST, can be NULL.
     dict_T *v_dict;  ///< Dictionary for VAR_DICT, can be NULL.
     partial_T *v_partial;  ///< Closure: function with args.
-  }           vval;  ///< Actual value.
+    blob_T *v_blob;  ///< Blob for VAR_BLOB, can be NULL.
+  } vval;  ///< Actual value.
 } typval_T;
 
 /// Values for (struct dictvar_S).dv_scope
@@ -146,8 +160,8 @@ typedef enum {
 typedef struct listitem_S listitem_T;
 
 struct listitem_S {
-  listitem_T  *li_next;  ///< Next item in list.
-  listitem_T  *li_prev;  ///< Previous item in list.
+  listitem_T *li_next;  ///< Next item in list.
+  listitem_T *li_prev;  ///< Previous item in list.
   typval_T li_tv;  ///< Item value.
 };
 
@@ -174,6 +188,8 @@ struct listvar_S {
   int lv_idx;  ///< Index of a cached item, used for optimising repeated l[idx].
   int lv_copyID;  ///< ID used by deepcopy().
   VarLockStatus lv_lock;  ///< Zero, VAR_LOCKED, VAR_FIXED.
+
+  LuaRef lua_table_ref;
 };
 
 // Static list with 10 items. Use tv_list_init_static10() to initialize.
@@ -183,25 +199,25 @@ typedef struct {
 } staticList10_T;
 
 #define TV_LIST_STATIC10_INIT { \
-    .sl_list = { \
-      .lv_first = NULL, \
-      .lv_last = NULL, \
-      .lv_refcount = 0, \
-      .lv_len = 0, \
-      .lv_watch = NULL, \
-      .lv_idx_item = NULL, \
-      .lv_lock = VAR_FIXED, \
-      .lv_used_next = NULL, \
-      .lv_used_prev = NULL, \
-    }, \
-  }
+  .sl_list = { \
+  .lv_first = NULL, \
+  .lv_last = NULL, \
+  .lv_refcount = 0, \
+  .lv_len = 0, \
+  .lv_watch = NULL, \
+  .lv_idx_item = NULL, \
+  .lv_lock = VAR_FIXED, \
+  .lv_used_next = NULL, \
+  .lv_used_prev = NULL, \
+  }, \
+}
 
 #define TV_DICTITEM_STRUCT(...) \
-    struct { \
-      typval_T di_tv;  /* Structure that holds scope dictionary itself. */ \
-      uint8_t di_flags;  /* Flags. */ \
-      char_u di_key[__VA_ARGS__];  /* Key value. */ \
-    }
+  struct { \
+    typval_T di_tv;  /* Structure that holds scope dictionary itself. */ \
+    uint8_t di_flags;  /* Flags. */ \
+    char_u di_key[__VA_ARGS__];  /* Key value. */ \
+  }
 
 /// Structure to hold a scope dictionary
 ///
@@ -239,6 +255,15 @@ struct dictvar_S {
   dict_T *dv_used_next;   ///< Next dictionary in used dictionaries list.
   dict_T *dv_used_prev;   ///< Previous dictionary in used dictionaries list.
   QUEUE watchers;         ///< Dictionary key watchers set by user code.
+
+  LuaRef lua_table_ref;
+};
+
+/// Structure to hold info about a Blob
+struct blobvar_S {
+  garray_T bv_ga;         ///< Growarray with the data.
+  int bv_refcount;        ///< Reference count.
+  VarLockStatus bv_lock;  ///< VAR_UNLOCKED, VAR_LOCKED, VAR_FIXED.
 };
 
 /// Type used for script ID
@@ -246,7 +271,7 @@ typedef int scid_T;
 /// Format argument for scid_T
 #define PRIdSCID "d"
 
-// SCript ConteXt (SCTX): identifies a script script line.
+// SCript ConteXt (SCTX): identifies a script line.
 // When sourcing a script "sc_lnum" is zero, "sourcing_lnum" is the current
 // line number. When executing a user function "sc_lnum" is the line where the
 // function was defined, "sourcing_lnum" is the line number inside the
@@ -258,43 +283,83 @@ typedef struct {
   linenr_T sc_lnum;  // line number
 } sctx_T;
 
+/// Maximum number of function arguments
+#define MAX_FUNC_ARGS   20
+/// Short variable name length
+#define VAR_SHORT_LEN 20
+/// Number of fixed variables used for arguments
+#define FIXVAR_CNT 12
+
+/// Callback interface for C function reference>
+///     Used for managing functions that were registered with |register_cfunc|
+typedef int (*cfunc_T)(int argcount, typval_T *argvars, typval_T *rettv, void *state);  // NOLINT
+/// Callback to clear cfunc_T and any associated state.
+typedef void (*cfunc_free_T)(void *state);
+
 // Structure to hold info for a function that is currently being executed.
 typedef struct funccall_S funccall_T;
 
-/// Structure to hold info for a user function.
-struct ufunc {
-  int          uf_varargs;       ///< variable nr of arguments
-  int          uf_flags;
-  int          uf_calls;         ///< nr of active calls
-  bool         uf_cleared;       ///< func_clear() was already called
-  garray_T     uf_args;          ///< arguments
-  garray_T     uf_lines;         ///< function lines
-  int          uf_profiling;     ///< true when func is being profiled
-  int          uf_prof_initialized;
-  // Profiling the function as a whole.
-  int          uf_tm_count;      ///< nr of calls
-  proftime_T   uf_tm_total;      ///< time spent in function + children
-  proftime_T   uf_tm_self;       ///< time spent in function itself
-  proftime_T   uf_tm_children;   ///< time spent in children this call
-  // Profiling the function per line.
-  int         *uf_tml_count;     ///< nr of times line was executed
-  proftime_T  *uf_tml_total;     ///< time spent in a line + children
-  proftime_T  *uf_tml_self;      ///< time spent in a line itself
-  proftime_T   uf_tml_start;     ///< start time for current line
-  proftime_T   uf_tml_children;  ///< time spent in children for this line
-  proftime_T   uf_tml_wait;      ///< start wait time for current line
-  int          uf_tml_idx;       ///< index of line being timed; -1 if none
-  int          uf_tml_execed;    ///< line being timed was executed
-  sctx_T       uf_script_ctx;    ///< SCTX where function was defined,
-                                 ///< used for s: variables
-  int          uf_refcount;      ///< reference count, see func_name_refcount()
-  funccall_T   *uf_scoped;       ///< l: local variables for closure
-  char_u       uf_name[];        ///< Name of function; can start with <SNR>123_
-                                 ///< (<SNR> is K_SPECIAL KS_EXTRA KE_SNR)
+struct funccall_S {
+  ufunc_T *func;  ///< Function being called.
+  int linenr;  ///< Next line to be executed.
+  int returned;  ///< ":return" used.
+  /// Fixed variables for arguments.
+  TV_DICTITEM_STRUCT(VAR_SHORT_LEN + 1) fixvar[FIXVAR_CNT];
+  dict_T l_vars;  ///< l: local function variables.
+  ScopeDictDictItem l_vars_var;  ///< Variable for l: scope.
+  dict_T l_avars;  ///< a: argument variables.
+  ScopeDictDictItem l_avars_var;  ///< Variable for a: scope.
+  list_T l_varlist;  ///< List for a:000.
+  listitem_T l_listitems[MAX_FUNC_ARGS];  ///< List items for a:000.
+  typval_T *rettv;  ///< Return value.
+  linenr_T breakpoint;  ///< Next line with breakpoint or zero.
+  int dbg_tick;  ///< Debug_tick when breakpoint was set.
+  int level;  ///< Top nesting level of executed function.
+  proftime_T prof_child;  ///< Time spent in a child.
+  funccall_T *caller;  ///< Calling function or NULL; or next funccal in
+                       ///< list pointed to by previous_funccal.
+  int fc_refcount;  ///< Number of user functions that reference this funccall.
+  int fc_copyID;  ///< CopyID used for garbage collection.
+  garray_T fc_funcs;  ///< List of ufunc_T* which keep a reference to "func".
 };
 
-/// Maximum number of function arguments
-#define MAX_FUNC_ARGS   20
+/// Structure to hold info for a user function.
+struct ufunc {
+  int uf_varargs;       ///< variable nr of arguments
+  int uf_flags;
+  int uf_calls;         ///< nr of active calls
+  bool uf_cleared;       ///< func_clear() was already called
+  garray_T uf_args;          ///< arguments
+  garray_T uf_def_args;      ///< default argument expressions
+  garray_T uf_lines;         ///< function lines
+  int uf_profiling;     ///< true when func is being profiled
+  int uf_prof_initialized;
+  // Managing cfuncs
+  cfunc_T uf_cb;            ///< C function extension callback
+  cfunc_free_T uf_cb_free;       ///< C function extension free callback
+  void *uf_cb_state;      ///< State of C function extension.
+  // Profiling the function as a whole.
+  int uf_tm_count;      ///< nr of calls
+  proftime_T uf_tm_total;      ///< time spent in function + children
+  proftime_T uf_tm_self;       ///< time spent in function itself
+  proftime_T uf_tm_children;   ///< time spent in children this call
+  // Profiling the function per line.
+  int *uf_tml_count;     ///< nr of times line was executed
+  proftime_T *uf_tml_total;     ///< time spent in a line + children
+  proftime_T *uf_tml_self;      ///< time spent in a line itself
+  proftime_T uf_tml_start;     ///< start time for current line
+  proftime_T uf_tml_children;  ///< time spent in children for this line
+  proftime_T uf_tml_wait;      ///< start wait time for current line
+  int uf_tml_idx;       ///< index of line being timed; -1 if none
+  int uf_tml_execed;    ///< line being timed was executed
+  sctx_T uf_script_ctx;    ///< SCTX where function was defined,
+                           ///< used for s: variables
+  int uf_refcount;      ///< reference count, see func_name_refcount()
+  funccall_T *uf_scoped;       ///< l: local variables for closure
+  char_u uf_name[];  ///< Name of function (actual size equals name);
+                     ///< can start with <SNR>123_
+                     ///< (<SNR> is K_SPECIAL KS_EXTRA KE_SNR)
+};
 
 struct partial_S {
   int pt_refcount;  ///< Reference count.
@@ -391,10 +456,8 @@ static inline void list_log(const list_T *const l,
 /// @param[in]  li1  List item 1.
 /// @param[in]  li2  List item 2, often used for integers and not list items.
 /// @param[in]  action  Logged action.
-static inline void list_log(const list_T *const l,
-                            const listitem_T *const li1,
-                            const listitem_T *const li2,
-                            const char *const action)
+static inline void list_log(const list_T *const l, const listitem_T *const li1,
+                            const listitem_T *const li2, const char *const action)
 {
   ListLog *tgt;
   if (list_log_first == NULL) {
@@ -423,7 +486,7 @@ static inline void list_log(const list_T *const l,
 
 /// Convert a hashitem pointer to a dictitem pointer
 #define TV_DICT_HI2DI(hi) \
-    ((dictitem_T *)((hi)->hi_key - offsetof(dictitem_T, di_key)))
+  ((dictitem_T *)((hi)->hi_key - offsetof(dictitem_T, di_key)))
 
 static inline void tv_list_ref(list_T *const l)
   REAL_FATTR_ALWAYS_INLINE;
@@ -444,7 +507,7 @@ static inline void tv_list_ref(list_T *const l)
 static inline void tv_list_set_ret(typval_T *const tv, list_T *const l)
   REAL_FATTR_ALWAYS_INLINE REAL_FATTR_NONNULL_ARG(1);
 
-/// Set a list as the return value
+/// Set a list as the return value.  Increments the reference count.
 ///
 /// @param[out]  tv  Object to receive the list
 /// @param[in,out]  l  List to pass to the object
@@ -477,8 +540,7 @@ static inline VarLockStatus tv_list_locked(const list_T *const l)
 ///
 /// @param[out]  l  List to modify.
 /// @param[in]  lock  New lock status.
-static inline void tv_list_set_lock(list_T *const l,
-                                    const VarLockStatus lock)
+static inline void tv_list_set_lock(list_T *const l, const VarLockStatus lock)
 {
   if (l == NULL) {
     assert(lock == VAR_FIXED);
@@ -493,8 +555,7 @@ static inline void tv_list_set_lock(list_T *const l,
 ///
 /// @param[out]  l  List to modify.
 /// @param[in]  copyid  New copyID.
-static inline void tv_list_set_copyid(list_T *const l,
-                                      const int copyid)
+static inline void tv_list_set_copyid(list_T *const l, const int copyid)
   FUNC_ATTR_NONNULL_ALL
 {
   l->lv_copyID = copyid;
@@ -660,6 +721,65 @@ static inline bool tv_dict_is_watched(const dict_T *const d)
   return d && !QUEUE_EMPTY(&d->watchers);
 }
 
+static inline void tv_blob_set_ret(typval_T *const tv, blob_T *const b)
+  REAL_FATTR_ALWAYS_INLINE REAL_FATTR_NONNULL_ARG(1);
+
+/// Set a blob as the return value.
+///
+/// Increments the reference count.
+///
+/// @param[out]  tv  Object to receive the blob.
+/// @param[in,out]  b  Blob to pass to the object.
+static inline void tv_blob_set_ret(typval_T *const tv, blob_T *const b)
+{
+  tv->v_type = VAR_BLOB;
+  tv->vval.v_blob = b;
+  if (b != NULL) {
+    b->bv_refcount++;
+  }
+}
+
+static inline int tv_blob_len(const blob_T *const b)
+  REAL_FATTR_PURE REAL_FATTR_WARN_UNUSED_RESULT;
+
+/// Get the length of the data in the blob, in bytes.
+///
+/// @param[in]  b  Blob to check.
+static inline int tv_blob_len(const blob_T *const b)
+{
+  if (b == NULL) {
+    return 0;
+  }
+  return b->bv_ga.ga_len;
+}
+
+static inline char_u tv_blob_get(const blob_T *const b, int idx)
+  REAL_FATTR_ALWAYS_INLINE REAL_FATTR_NONNULL_ALL REAL_FATTR_WARN_UNUSED_RESULT;
+
+/// Get the byte at index `idx` in the blob.
+///
+/// @param[in]  b  Blob to index. Cannot be NULL.
+/// @param[in]  idx  Index in a blob. Must be valid.
+///
+/// @return Byte value at the given index.
+static inline char_u tv_blob_get(const blob_T *const b, int idx)
+{
+  return ((char_u *)b->bv_ga.ga_data)[idx];
+}
+
+static inline void tv_blob_set(blob_T *const b, int idx, char_u c)
+  REAL_FATTR_ALWAYS_INLINE REAL_FATTR_NONNULL_ALL;
+
+/// Store the byte `c` at index `idx` in the blob.
+///
+/// @param[in]  b  Blob to index. Cannot be NULL.
+/// @param[in]  idx  Index in a blob. Must be valid.
+/// @param[in]  c  Value to store.
+static inline void tv_blob_set(blob_T *const b, int idx, char_u c)
+{
+  ((char_u *)b->bv_ga.ga_data)[idx] = c;
+}
+
 /// Initialize VimL object
 ///
 /// Initializes to unlocked VAR_UNKNOWN object.
@@ -673,10 +793,10 @@ static inline void tv_init(typval_T *const tv)
 }
 
 #define TV_INITIAL_VALUE \
-    ((typval_T) { \
-      .v_type = VAR_UNKNOWN, \
-      .v_lock = VAR_UNLOCKED, \
-    })
+  ((typval_T) { \
+    .v_type = VAR_UNKNOWN, \
+    .v_lock = VAR_UNLOCKED, \
+  })
 
 /// Empty string
 ///
@@ -695,16 +815,16 @@ extern bool tv_in_free_unref_items;
 /// @param  li  Name of the variable with current listitem_T entry.
 /// @param  code  Cycle body.
 #define _TV_LIST_ITER_MOD(modifier, l, li, code) \
-    do { \
-      modifier list_T *const l_ = (l); \
-      list_log(l_, NULL, NULL, "iter" #modifier); \
-      if (l_ != NULL) { \
-        for (modifier listitem_T *li = l_->lv_first; \
-             li != NULL; li = li->li_next) { \
-          code \
-        } \
+  do { \
+    modifier list_T *const l_ = (l); \
+    list_log(l_, NULL, NULL, "iter" #modifier); \
+    if (l_ != NULL) { \
+      for (modifier listitem_T *li = l_->lv_first; \
+           li != NULL; li = li->li_next) { \
+        code \
       } \
-    } while (0)
+    } \
+  } while (0)
 
 /// Iterate over a list
 ///
@@ -715,7 +835,7 @@ extern bool tv_in_free_unref_items;
 /// @param  li  Name of the variable with current listitem_T entry.
 /// @param  code  Cycle body.
 #define TV_LIST_ITER(l, li, code) \
-    _TV_LIST_ITER_MOD(, l, li, code)
+  _TV_LIST_ITER_MOD( , l, li, code)
 
 /// Iterate over a list
 ///
@@ -726,7 +846,7 @@ extern bool tv_in_free_unref_items;
 /// @param  li  Name of the variable with current listitem_T entry.
 /// @param  code  Cycle body.
 #define TV_LIST_ITER_CONST(l, li, code) \
-    _TV_LIST_ITER_MOD(const, l, li, code)
+  _TV_LIST_ITER_MOD(const, l, li, code)
 
 // Below macros are macros to avoid duplicating code for functionally identical
 // const and non-const function variants.
@@ -763,21 +883,21 @@ extern bool tv_in_free_unref_items;
 /// @param  di  Name of the variable with current dictitem_T entry.
 /// @param  code  Cycle body.
 #define TV_DICT_ITER(d, di, code) \
-    HASHTAB_ITER(&(d)->dv_hashtab, di##hi_, { \
+  HASHTAB_ITER(&(d)->dv_hashtab, di##hi_, { \
+    { \
+      dictitem_T *const di = TV_DICT_HI2DI(di##hi_); \
       { \
-        dictitem_T *const di = TV_DICT_HI2DI(di##hi_); \
-        { \
-          code \
-        } \
+        code \
       } \
-    })
+    } \
+  })
 
 static inline bool tv_get_float_chk(const typval_T *const tv,
                                     float_T *const ret_f)
   REAL_FATTR_NONNULL_ALL REAL_FATTR_WARN_UNUSED_RESULT;
 
 // FIXME circular dependency, cannot import message.h.
-bool emsgf(const char *const fmt, ...);
+bool semsg(const char *const fmt, ...);
 
 /// Get the float value
 ///
@@ -787,8 +907,7 @@ bool emsgf(const char *const fmt, ...);
 /// @param[out]  ret_f  Location where resulting float is stored.
 ///
 /// @return true in case of success, false if tv is not a number or float.
-static inline bool tv_get_float_chk(const typval_T *const tv,
-                                    float_T *const ret_f)
+static inline bool tv_get_float_chk(const typval_T *const tv, float_T *const ret_f)
 {
   if (tv->v_type == VAR_FLOAT) {
     *ret_f = tv->vval.v_float;
@@ -798,7 +917,7 @@ static inline bool tv_get_float_chk(const typval_T *const tv,
     *ret_f = (float_T)tv->vval.v_number;
     return true;
   }
-  emsgf(_("E808: Number or Float required"));
+  semsg("%s", _("E808: Number or Float required"));
   return false;
 }
 
