@@ -392,6 +392,186 @@ describe('api/buf', function()
     end)
   end)
 
+  describe('nvim_buf_get_lines, nvim_buf_set_text', function()
+    local get_lines, set_text = curbufmeths.get_lines, curbufmeths.set_text
+
+    it('works', function()
+      insert([[
+      hello foo!
+      text
+      ]])
+
+      eq({'hello foo!'}, get_lines(0, 1, true))
+
+
+      -- can replace a single word
+      set_text(0, 6, 0, 9, {'world'})
+      eq({'hello world!', 'text'}, get_lines(0, 2, true))
+
+      -- can insert text
+      set_text(0, 0, 0, 0, {'well '})
+      eq({'well hello world!', 'text'}, get_lines(0, 2, true))
+
+      -- can delete text
+      set_text(0, 0, 0, 5, {''})
+      eq({'hello world!', 'text'}, get_lines(0, 2, true))
+
+      -- can replace with multiple lines
+      set_text(0, 6, 0, 11, {'foo', 'wo', 'more'})
+      eq({'hello foo', 'wo', 'more!', 'text'}, get_lines(0,  4, true))
+
+      -- will join multiple lines if needed
+      set_text(0, 6, 3, 4, {'bar'})
+      eq({'hello bar'}, get_lines(0,  1, true))
+
+      -- can use negative line numbers
+      set_text(-2, 0, -2, 5, {'goodbye'})
+      eq({'goodbye bar', ''}, get_lines(0, -1, true))
+
+      set_text(-1, 0, -1, 0, {'text'})
+      eq({'goodbye bar', 'text'}, get_lines(0, 2, true))
+    end)
+
+    it('works with undo', function()
+        insert([[
+        hello world!
+        foo bar
+        ]])
+
+        -- setting text
+        set_text(0, 0, 0, 0, {'well '})
+        feed('u')
+        eq({'hello world!'}, get_lines(0, 1, true))
+
+        -- deleting text
+        set_text(0, 0, 0, 6, {''})
+        feed('u')
+        eq({'hello world!'}, get_lines(0, 1, true))
+
+        -- inserting newlines
+        set_text(0, 0, 0, 0, {'hello', 'mr '})
+        feed('u')
+        eq({'hello world!'}, get_lines(0, 1, true))
+
+        -- deleting newlines
+        set_text(0, 0, 1, 4, {'hello'})
+        feed('u')
+        eq({'hello world!'}, get_lines(0, 1, true))
+    end)
+
+    it('updates the cursor position', function()
+      insert([[
+      hello world!
+      ]])
+
+      -- position the cursor on `!`
+      curwin('set_cursor', {1, 11})
+      -- replace 'world' with 'foo'
+      set_text(0, 6, 0, 11, {'foo'})
+      eq('hello foo!', curbuf_depr('get_line', 0))
+      -- cursor should be moved left by two columns (replacement is shorter by 2 chars)
+      eq({1, 9}, curwin('get_cursor'))
+    end)
+
+    it('can handle NULs', function()
+      set_text(0, 0, 0, 0, {'ab\0cd'})
+      eq('ab\0cd', curbuf_depr('get_line', 0))
+    end)
+
+    it('adjusts extmarks', function()
+      local ns = request('nvim_create_namespace', "my-fancy-plugin")
+      insert([[
+      foo bar
+      baz
+      ]])
+      local id1 = curbufmeths.set_extmark(ns, 0, 1, {})
+      local id2 = curbufmeths.set_extmark(ns, 0, 7, {})
+      local id3 = curbufmeths.set_extmark(ns, 1, 1, {})
+      set_text(0, 4, 0, 7, {"q"})
+
+      eq({'foo q', 'baz'}, get_lines(0, 2, true))
+      -- mark before replacement point is unaffected
+      eq({0, 1}, curbufmeths.get_extmark_by_id(ns, id1, {}))
+      -- mark gets shifted back because the replacement was shorter
+      eq({0, 5}, curbufmeths.get_extmark_by_id(ns, id2, {}))
+      -- mark on the next line is unaffected
+      eq({1, 1}, curbufmeths.get_extmark_by_id(ns, id3, {}))
+
+      -- replacing the text spanning two lines will adjust the mark on the next line
+      set_text(0, 3, 1, 3, {"qux"})
+      eq({'fooqux', ''}, get_lines(0, 2, true))
+      eq({0, 6}, curbufmeths.get_extmark_by_id(ns, id3, {}))
+      -- but mark before replacement point is still unaffected
+      eq({0, 1}, curbufmeths.get_extmark_by_id(ns, id1, {}))
+      -- and the mark in the middle was shifted to the end of the insertion
+      eq({0, 6}, curbufmeths.get_extmark_by_id(ns, id2, {}))
+
+      -- marks should be put back into the same place after undoing
+      set_text(0, 0, 0, 2, {''})
+      feed('u')
+      eq({0, 1}, curbufmeths.get_extmark_by_id(ns, id1, {}))
+      eq({0, 6}, curbufmeths.get_extmark_by_id(ns, id2, {}))
+      eq({0, 6}, curbufmeths.get_extmark_by_id(ns, id3, {}))
+
+	  -- marks should be shifted over by the correct number of bytes for multibyte
+	  -- chars
+	  set_text(0, 0, 0, 0, {'Ø'})
+	  eq({0, 3}, curbufmeths.get_extmark_by_id(ns, id1, {}))
+	  eq({0, 8}, curbufmeths.get_extmark_by_id(ns, id2, {}))
+	  eq({0, 8}, curbufmeths.get_extmark_by_id(ns, id3, {}))
+    end)
+
+    it("correctly marks changed region for redraw #13890", function()
+      local screen = Screen.new(20, 5)
+      screen:attach()
+
+      insert([[
+      AAA
+      BBB
+      ]])
+
+      curbufmeths.set_text(0, 0, 1, 3, {'XXX', 'YYY'})
+
+      screen:expect([[
+  XXX                 |
+  YYY                 |
+  ^                    |
+  ~                   |
+                      |
+
+      ]])
+
+    end)
+  end)
+
+  describe('nvim_buf_get_text', function()
+    local get_text = curbufmeths.get_text
+
+    it('works', function()
+      insert([[
+      hello foo!
+      text]])
+
+      eq({'hello'}, get_text(0, 0, 0, 5, {}))
+      eq({'hello foo!'}, get_text(0, 0, 0, 42, {}))
+      eq({'foo!'}, get_text(0, 6, 0, 10, {}))
+      eq({'foo!', 'tex'}, get_text(0, 6, 1, 3, {}))
+      eq({'foo!', 'tex'}, get_text(-2, 6, -1, 3, {}))
+      eq({''}, get_text(0, 18, 0, 20, {}))
+      eq({'ext'}, get_text(-1, 1, -1, 4, {}))
+    end)
+
+    it('errors on out-of-range', function()
+      eq(false, pcall(get_text, 2, 0, 3, 0, {}))
+      eq(false, pcall(get_text, 0, 0, 4, 0, {}))
+    end)
+
+    it('errors when start is greater than end', function()
+      eq(false, pcall(get_text, 1, 0, 0, 0, {}))
+      eq(false, pcall(get_text, 0, 1, 0, 0, {}))
+    end)
+  end)
+
   describe('nvim_buf_get_offset', function()
     local get_offset = curbufmeths.get_offset
     it('works', function()
@@ -484,6 +664,13 @@ describe('api/buf', function()
       -- Doesn't change the global value
       eq([[^\s*#\s*define]], nvim('get_option', 'define'))
     end)
+
+    it('returns values for unset local options', function()
+      -- 'undolevels' is only set to its "unset" value when a new buffer is
+      -- created
+      command('enew')
+      eq(-123456, curbuf('get_option', 'undolevels'))
+    end)
   end)
 
   describe('nvim_buf_get_name, nvim_buf_set_name', function()
@@ -534,6 +721,26 @@ describe('api/buf', function()
     end)
   end)
 
+  describe('nvim_buf_delete', function()
+    it('allows for just deleting', function()
+      nvim('command', 'new')
+      local b = nvim('get_current_buf')
+      ok(buffer('is_valid', b))
+      nvim('buf_delete', b, {})
+      ok(not buffer('is_loaded', b))
+      ok(not buffer('is_valid', b))
+    end)
+
+    it('allows for just unloading', function()
+      nvim('command', 'new')
+      local b = nvim('get_current_buf')
+      ok(buffer('is_valid', b))
+      nvim('buf_delete', b, { unload = true })
+      ok(not buffer('is_loaded', b))
+      ok(buffer('is_valid', b))
+    end)
+  end)
+
   describe('nvim_buf_get_mark', function()
     it('works', function()
       curbuf('set_lines', -1, -1, true, {'a', 'bit of', 'text'})
@@ -542,4 +749,59 @@ describe('api/buf', function()
       eq({3, 0}, curbuf('get_mark', 'v'))
     end)
   end)
+
+  describe('nvim_buf_set_mark', function()
+    it('works with buffer local marks', function()
+      curbufmeths.set_lines(-1, -1, true, {'a', 'bit of', 'text'})
+      eq(true, curbufmeths.set_mark('z', 1, 1, {}))
+      eq({1, 1}, curbufmeths.get_mark('z'))
+    end)
+    it('works with file/uppercase marks', function()
+      curbufmeths.set_lines(-1, -1, true, {'a', 'bit of', 'text'})
+      eq(true, curbufmeths.set_mark('Z', 3, 1, {}))
+      eq({3, 1}, curbufmeths.get_mark('Z'))
+    end)
+    it('fails when invalid marks names are used', function()
+      eq(false, pcall(curbufmeths.set_mark, '!', 1, 0, {}))
+      eq(false, pcall(curbufmeths.set_mark, 'fail', 1, 0, {}))
+    end)
+    it('fails when invalid buffer number is used', function()
+      eq(false, pcall(meths.buf_set_mark, 99, 'a', 1, 1, {}))
+    end)
+  end)
+
+  describe('nvim_buf_del_mark', function()
+    it('works with buffer local marks', function()
+      curbufmeths.set_lines(-1, -1, true, {'a', 'bit of', 'text'})
+      curbufmeths.set_mark('z', 3, 1, {})
+      eq(true, curbufmeths.del_mark('z'))
+      eq({0, 0}, curbufmeths.get_mark('z'))
+    end)
+    it('works with file/uppercase marks', function()
+      curbufmeths.set_lines(-1, -1, true, {'a', 'bit of', 'text'})
+      curbufmeths.set_mark('Z', 3, 3, {})
+      eq(true, curbufmeths.del_mark('Z'))
+      eq({0, 0}, curbufmeths.get_mark('Z'))
+    end)
+    it('returns false in marks not set in this buffer', function()
+      local abuf = meths.create_buf(false,true)
+      bufmeths.set_lines(abuf, -1, -1, true, {'a', 'bit of', 'text'})
+      bufmeths.set_mark(abuf, 'A', 2, 2, {})
+      eq(false, curbufmeths.del_mark('A'))
+      eq({2, 2}, bufmeths.get_mark(abuf, 'A'))
+    end)
+    it('returns false if mark was not deleted', function()
+      curbufmeths.set_lines(-1, -1, true, {'a', 'bit of', 'text'})
+      curbufmeths.set_mark('z', 3, 1, {})
+      eq(true, curbufmeths.del_mark('z'))
+      eq(false, curbufmeths.del_mark('z'))  -- Mark was already deleted
+    end)
+    it('fails when invalid marks names are used', function()
+      eq(false, pcall(curbufmeths.del_mark, '!'))
+      eq(false, pcall(curbufmeths.del_mark, 'fail'))
+    end)
+    it('fails when invalid buffer number is used', function()
+      eq(false, pcall(meths.buf_del_mark, 99, 'a'))
+    end)
+  end)
 end)
