@@ -3,6 +3,8 @@ local Screen = require('test.functional.ui.screen')
 local clear, feed = helpers.clear, helpers.feed
 local source = helpers.source
 local command = helpers.command
+local assert_alive = helpers.assert_alive
+local uname = helpers.uname
 
 local function new_screen(opt)
   local screen = Screen.new(25, 5)
@@ -13,6 +15,8 @@ local function new_screen(opt)
     [3] = {bold = true, reverse = true},
     [4] = {foreground = Screen.colors.Grey100, background = Screen.colors.Red},
     [5] = {bold = true, foreground = Screen.colors.SeaGreen4},
+    [6] = {foreground = Screen.colors.Magenta},
+    [7] = {bold = true, foreground = Screen.colors.Brown},
   })
   return screen
 end
@@ -25,10 +29,6 @@ local function test_cmdline(linegrid)
     screen = new_screen({rgb=true, ext_cmdline=true, ext_linegrid=linegrid})
   end)
 
-  after_each(function()
-    screen:detach()
-  end)
-
   it('works', function()
     feed(':')
     screen:expect{grid=[[
@@ -270,7 +270,7 @@ local function test_cmdline(linegrid)
       special = {'"', true},
     }, {
       firstc = "=",
-      content = {{"1"}, {"+"}, {"2"}},
+      content = {{"1", 6}, {"+", 7}, {"2", 6}},
       pos = 3,
     }}
 
@@ -762,6 +762,7 @@ local function test_cmdline(linegrid)
   end)
 
   it("doesn't send invalid events when aborting mapping #10000", function()
+    command('set notimeout')
     command('cnoremap ab c')
 
     feed(':xa')
@@ -779,7 +780,7 @@ local function test_cmdline(linegrid)
     }}}
 
     -- This used to send an invalid event where pos where larger than the total
-    -- lenght of content. Checked in _handle_cmdline_show.
+    -- length of content. Checked in _handle_cmdline_show.
     feed('<esc>')
     screen:expect([[
       ^                         |
@@ -804,10 +805,6 @@ describe('cmdline redraw', function()
     screen = new_screen({rgb=true})
   end)
 
-  after_each(function()
-    screen:detach()
-  end)
-
   it('with timer', function()
     feed(':012345678901234567890123456789')
     screen:expect{grid=[[
@@ -828,9 +825,8 @@ describe('cmdline redraw', function()
   end)
 
   it('with <Cmd>', function()
-    if 'openbsd' == helpers.uname() then
-      pending('FIXME #10804', function() end)
-      return
+    if string.find(uname(), 'bsd') then
+      pending('FIXME #10804')
     end
     command('cmap a <Cmd>call sin(0)<CR>')  -- no-op
     feed(':012345678901234567890123456789')
@@ -851,3 +847,14 @@ describe('cmdline redraw', function()
     ]], unchanged=true}
   end)
 end)
+
+describe("cmdline height", function()
+  it("does not crash resized screen #14263", function()
+    clear()
+    local screen = Screen.new(25, 10)
+    screen:attach()
+    command('set cmdheight=9999')
+    screen:try_resize(25, 5)
+    assert_alive()
+  end)
+end)
