@@ -14,10 +14,11 @@ local neq = helpers.neq
 local ok = helpers.ok
 local retry = helpers.retry
 local source = helpers.source
-local wait = helpers.wait
+local poke_eventloop = helpers.poke_eventloop
 local nvim = helpers.nvim
 local sleep = helpers.sleep
 local nvim_dir = helpers.nvim_dir
+local assert_alive = helpers.assert_alive
 
 local default_text = [[
   Inc substitution on
@@ -84,18 +85,19 @@ local function common_setup(screen, inccommand, text)
       [14] = {foreground = Screen.colors.White, background = Screen.colors.Red},
       [15] = {bold=true, foreground=Screen.colors.Blue},
       [16] = {background=Screen.colors.Grey90},  -- cursorline
+      [17] = {foreground = Screen.colors.Blue1},
       vis  = {background=Screen.colors.LightGrey}
     })
   end
 
-  command("set inccommand=" .. (inccommand and inccommand or ""))
+  command("set inccommand=" .. (inccommand or ""))
 
   if text then
     insert(text)
   end
 end
 
-describe(":substitute, inccommand=split", function()
+describe(":substitute, inccommand=split interactivity", function()
   before_each(function()
     clear()
     common_setup(nil, "split", default_text)
@@ -112,7 +114,7 @@ describe(":substitute, inccommand=split", function()
 
   it("no preview if invoked by a script", function()
     source('%s/tw/MO/g')
-    wait()
+    poke_eventloop()
     eq(1, eval("bufnr('$')"))
     -- sanity check: assert the buffer state
     expect(default_text:gsub("tw", "MO"))
@@ -121,10 +123,10 @@ describe(":substitute, inccommand=split", function()
   it("no preview if invoked by feedkeys()", function()
     -- in a script...
     source([[:call feedkeys(":%s/tw/MO/g\<CR>")]])
-    wait()
+    poke_eventloop()
     -- or interactively...
     feed([[:call feedkeys(":%s/tw/MO/g\<CR>")<CR>]])
-    wait()
+    poke_eventloop()
     eq(1, eval("bufnr('$')"))
     -- sanity check: assert the buffer state
     expect(default_text:gsub("tw", "MO"))
@@ -160,7 +162,7 @@ describe(":substitute, 'inccommand' preserves", function()
       insert(default_text)
       feed_command("set inccommand=" .. case)
 
-      local delims = { '/', '#', ';', '%', ',', '@', '!', ''}
+      local delims = { '/', '#', ';', '%', ',', '@', '!' }
       for _,delim in pairs(delims) do
         feed_command("%s"..delim.."lines"..delim.."LINES"..delim.."g")
         expect([[
@@ -192,7 +194,7 @@ describe(":substitute, 'inccommand' preserves", function()
 
       -- Start typing an incomplete :substitute command.
       feed([[:%s/e/YYYY/g]])
-      wait()
+      poke_eventloop()
       -- Cancel the :substitute.
       feed([[<C-\><C-N>]])
 
@@ -228,7 +230,7 @@ describe(":substitute, 'inccommand' preserves", function()
 
       -- Start typing an incomplete :substitute command.
       feed([[:%s/e/YYYY/g]])
-      wait()
+      poke_eventloop()
       -- Cancel the :substitute.
       feed([[<C-\><C-N>]])
 
@@ -249,7 +251,7 @@ describe(":substitute, 'inccommand' preserves", function()
         some text 1
         some text 2]])
       feed(":%s/e/XXX/")
-      wait()
+      poke_eventloop()
 
       eq(expected_tick, eval("b:changedtick"))
     end)
@@ -291,6 +293,70 @@ describe(":substitute, 'inccommand' preserves", function()
     end)
   end
 
+  for _, case in ipairs({'', 'split', 'nosplit'}) do
+    it('previous substitute string ~ (inccommand='..case..') #12109', function()
+      local screen = Screen.new(30,10)
+      common_setup(screen, case, default_text)
+
+      feed(':%s/Inc/SUB<CR>')
+      expect([[
+        SUB substitution on
+        two lines
+        ]])
+
+      feed(':%s/line/')
+      poke_eventloop()
+      feed('~')
+      poke_eventloop()
+      feed('<CR>')
+      expect([[
+        SUB substitution on
+        two SUBs
+        ]])
+
+      feed(':%s/sti/')
+      poke_eventloop()
+      feed('~')
+      poke_eventloop()
+      feed('B')
+      poke_eventloop()
+      feed('<CR>')
+      expect([[
+        SUB subSUBBtution on
+        two SUBs
+        ]])
+
+      feed(':%s/ion/NEW<CR>')
+      expect([[
+        SUB subSUBBtutNEW on
+        two SUBs
+        ]])
+
+      feed(':%s/two/')
+      poke_eventloop()
+      feed('N')
+      poke_eventloop()
+      feed('~')
+      poke_eventloop()
+      feed('<CR>')
+      expect([[
+        SUB subSUBBtutNEW on
+        NNEW SUBs
+        ]])
+
+      feed(':%s/bS/')
+      poke_eventloop()
+      feed('~')
+      poke_eventloop()
+      feed('W')
+      poke_eventloop()
+      feed('<CR>')
+      expect([[
+        SUB suNNEWWUBBtutNEW on
+        NNEW SUBs
+        ]])
+    end)
+  end
 end)
 
 describe(":substitute, 'inccommand' preserves undo", function()
@@ -556,7 +622,6 @@ describe(":substitute, 'inccommand' preserves undo", function()
         ]])
       end
     end
-    screen:detach()
   end)
 
   it('with undolevels=2', function()
@@ -647,7 +712,6 @@ describe(":substitute, 'inccommand' preserves undo", function()
           Already ...t change |
         ]])
       end
-      screen:detach()
     end
   end)
 
@@ -713,7 +777,6 @@ describe(":substitute, 'inccommand' preserves undo", function()
         Already ...t change |
       ]])
     end
-    screen:detach()
   end)
 
 end)
@@ -726,19 +789,15 @@ describe(":substitute, inccommand=split", function()
     common_setup(screen, "split", default_text .. default_text)
   end)
 
-  after_each(function()
-    screen:detach()
-  end)
-
   it("preserves 'modified' buffer flag", function()
     feed_command("set nomodified")
     feed(":%s/tw")
     screen:expect([[
+      Inc substitution on           |
+      {12:tw}o lines                     |
       Inc substitution on           |
       {12:tw}o lines                     |
                                     |
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name]                     }|
       |2| {12:tw}o lines                 |
       |4| {12:tw}o lines                 |
@@ -786,16 +845,69 @@ describe(":substitute, inccommand=split", function()
       {15:~                             }|
       :silent tabedit %s/tw/to^      |
     ]])
+    feed('<Esc>')
+
+    -- leading colons
+    feed(':::%s/tw/to')
+    screen:expect{any=[[{12:to}o lines]]}
+    feed('<Esc>')
+    screen:expect{any=[[two lines]]}
+  end)
+
+  it("ignores new-window modifiers when splitting the preview window", function()
+    -- one modifier
+    feed(':topleft %s/tw/to')
+    screen:expect([[
+      Inc substitution on           |
+      {12:to}o lines                     |
+      Inc substitution on           |
+      {12:to}o lines                     |
+                                    |
+      {11:[No Name] [+]                 }|
+      |2| {12:to}o lines                 |
+      |4| {12:to}o lines                 |
+      {15:~                             }|
+      {15:~                             }|
+      {15:~                             }|
+      {15:~                             }|
+      {15:~                             }|
+      {10:[Preview]                     }|
+      :topleft %s/tw/to^             |
+    ]])
+    feed('<Esc>')
+    screen:expect{any=[[two lines]]}
+
+    -- multiple modifiers
+    feed(':topleft vert %s/tw/to')
+    screen:expect([[
+      Inc substitution on           |
+      {12:to}o lines                     |
+      Inc substitution on           |
+      {12:to}o lines                     |
+                                    |
+      {11:[No Name] [+]                 }|
+      |2| {12:to}o lines                 |
+      |4| {12:to}o lines                 |
+      {15:~                             }|
+      {15:~                             }|
+      {15:~                             }|
+      {15:~                             }|
+      {15:~                             }|
+      {10:[Preview]                     }|
+      :topleft vert %s/tw/to^        |
+    ]])
+    feed('<Esc>')
+    screen:expect{any=[[two lines]]}
   end)
 
   it('shows split window when typing the pattern', function()
     feed(":%s/tw")
     screen:expect([[
+      Inc substitution on           |
+      {12:tw}o lines                     |
       Inc substitution on           |
       {12:tw}o lines                     |
                                     |
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |2| {12:tw}o lines                 |
       |4| {12:tw}o lines                 |
@@ -812,11 +924,11 @@ describe(":substitute, inccommand=split", function()
   it('shows preview with empty replacement', function()
     feed(":%s/tw/")
     screen:expect([[
+      Inc substitution on           |
+      o lines                       |
       Inc substitution on           |
       o lines                       |
                                     |
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |2| o lines                   |
       |4| o lines                   |
@@ -831,11 +943,11 @@ describe(":substitute, inccommand=split", function()
 
     feed("x")
     screen:expect([[
+      Inc substitution on           |
+      {12:x}o lines                      |
       Inc substitution on           |
       {12:x}o lines                      |
                                     |
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |2| {12:x}o lines                  |
       |4| {12:x}o lines                  |
@@ -850,11 +962,11 @@ describe(":substitute, inccommand=split", function()
 
     feed("<bs>")
     screen:expect([[
+      Inc substitution on           |
+      o lines                       |
       Inc substitution on           |
       o lines                       |
                                     |
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |2| o lines                   |
       |4| o lines                   |
@@ -872,11 +984,11 @@ describe(":substitute, inccommand=split", function()
   it('shows split window when typing replacement', function()
     feed(":%s/tw/XX")
     screen:expect([[
+      Inc substitution on           |
+      {12:XX}o lines                     |
       Inc substitution on           |
       {12:XX}o lines                     |
                                     |
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |2| {12:XX}o lines                 |
       |4| {12:XX}o lines                 |
@@ -938,11 +1050,11 @@ describe(":substitute, inccommand=split", function()
     feed(":%s/tw")
     -- 'cursorline' is NOT active during preview.
     screen:expect([[
+      Inc substitution on           |
       {12:tw}o lines                     |
       Inc substitution on           |
       {12:tw}o lines                     |
                                     |
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |2| {12:tw}o lines                 |
       |4| {12:tw}o lines                 |
@@ -1080,15 +1192,15 @@ describe(":substitute, inccommand=split", function()
     feed(":%s/tw/Xo/g")
     -- Delete and re-type the g a few times.
     feed("<BS>")
-    wait()
+    poke_eventloop()
     feed("g")
-    wait()
+    poke_eventloop()
     feed("<BS>")
-    wait()
+    poke_eventloop()
     feed("g")
-    wait()
+    poke_eventloop()
     feed("<CR>")
-    wait()
+    poke_eventloop()
     feed(":vs tmp<enter>")
     eq(3, helpers.call('bufnr', '$'))
   end)
@@ -1123,7 +1235,7 @@ describe(":substitute, inccommand=split", function()
     feed_command("silent edit! test/functional/fixtures/bigfile_oneline.txt")
     -- Start :substitute with a slow pattern.
     feed([[:%s/B.*N/x]])
-    wait()
+    poke_eventloop()
 
     -- Assert that 'inccommand' is DISABLED in cmdline mode.
     eq("", eval("&inccommand"))
@@ -1241,10 +1353,6 @@ describe("inccommand=nosplit", function()
     common_setup(screen, "nosplit", default_text .. default_text)
   end)
 
-  after_each(function()
-    if screen then screen:detach() end
-  end)
-
   it("works with :smagic, :snomagic", function()
     feed_command("set hlsearch")
     insert("Line *.3.* here")
@@ -1316,7 +1424,7 @@ describe("inccommand=nosplit", function()
     feed("<Esc>")
     command("set icm=nosplit")
     feed(":%s/tw/OKOK")
-    wait()
+    poke_eventloop()
     screen:expect([[
       Inc substitution on |
       {12:OKOK}o lines         |
@@ -1443,6 +1551,29 @@ describe("inccommand=nosplit", function()
     ]])
     eq(eval('v:null'), eval('v:exiting'))
   end)
+
+  it("does not break bar-separated command #8796", function()
+    source([[
+      function! F()
+        if v:false | return | endif
+      endfun
+    ]])
+    command('call timer_start(10, {-> F()}, {"repeat":-1})')
+    feed(':%s/')
+    sleep(20)  -- Allow some timer activity.
+    screen:expect([[
+      Inc substitution on |
+      two lines           |
+      Inc substitution on |
+      two lines           |
+                          |
+      {15:~                   }|
+      {15:~                   }|
+      {15:~                   }|
+      {15:~                   }|
+      :%s/^                |
+    ]])
+  end)
 end)
 
 describe(":substitute, 'inccommand' with a failing expression", function()
@@ -1719,10 +1850,6 @@ describe("'inccommand' split windows", function()
     common_setup(screen, "split", default_text)
   end
 
-  after_each(function()
-    screen:detach()
-  end)
-
   it('work after more splits', function()
     refresh()
 
@@ -2205,11 +2332,11 @@ describe(":substitute", function()
 
     feed("/KKK")
     screen:expect([[
+      T T123 T T123 T2T TT T23423424|
+      x                             |
       afa {12:KKK}adf la;lkd {12:KKK}alx      |
                                     |
       {15:~                             }|
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |3| afa {12:KKK}adf la;lkd {12:KKK}alx  |
       {15:~                             }|
@@ -2253,6 +2380,76 @@ describe(":substitute", function()
     ]])
   end)
 
+  it("inccommand=split, contraction of two subsequent NL chars", function()
+    -- luacheck: push ignore 611
+    local text = [[
+      AAA AA
+      
+      BBB BB
+      
+      CCC CC
+      
+]]
+    -- luacheck: pop
+
+    -- This used to crash, but more than 20 highlight entries are required
+    -- to reproduce it (so that the marktree has multiple nodes)
+    common_setup(screen, "split", string.rep(text,10))
+    feed(":%s/\\n\\n/<c-v><c-m>/g")
+    screen:expect{grid=[[
+      CCC CC                        |
+      AAA AA                        |
+      BBB BB                        |
+      CCC CC                        |
+                                    |
+      {11:[No Name] [+]                 }|
+      | 1| AAA AA                   |
+      | 2|{12: }BBB BB                   |
+      | 3|{12: }CCC CC                   |
+      | 4|{12: }AAA AA                   |
+      | 5|{12: }BBB BB                   |
+      | 6|{12: }CCC CC                   |
+      | 7|{12: }AAA AA                   |
+      {10:[Preview]                     }|
+      :%s/\n\n/{17:^M}/g^                 |
+    ]]}
+    assert_alive()
+  end)
+
+  it("inccommand=nosplit, contraction of two subsequent NL chars", function()
+    -- luacheck: push ignore 611
+    local text = [[
+      AAA AA
+      
+      BBB BB
+      
+      CCC CC
+      
+]]
+    -- luacheck: pop
+
+    common_setup(screen, "nosplit", string.rep(text,10))
+    feed(":%s/\\n\\n/<c-v><c-m>/g")
+    screen:expect{grid=[[
+      CCC CC                        |
+      AAA AA                        |
+      BBB BB                        |
+      CCC CC                        |
+      AAA AA                        |
+      BBB BB                        |
+      CCC CC                        |
+      AAA AA                        |
+      BBB BB                        |
+      CCC CC                        |
+      AAA AA                        |
+      BBB BB                        |
+      CCC CC                        |
+                                    |
+      :%s/\n\n/{17:^M}/g^                 |
+    ]]}
+    assert_alive()
+  end)
+
   it("inccommand=split, multibyte text", function()
     common_setup(screen, "split", multibyte_text)
     feed(":%s/£.*ѫ/X¥¥")
@@ -2482,14 +2679,14 @@ describe(":substitute", function()
 
     feed("<C-c>")
     feed('gg')
-    wait()
+    poke_eventloop()
     feed([[:%s/\(some\)\@<lt>!thing/one/]])
     screen:expect([[
+      something                     |
       every{12:one}                      |
       someone                       |
       {15:~                             }|
       {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |2| every{12:one}                  |
       {15:~                             }|
@@ -2503,7 +2700,7 @@ describe(":substitute", function()
     ]])
 
     feed([[<C-c>]])
-    wait()
+    poke_eventloop()
     feed([[:%s/some\(thing\)\@=/every/]])
     screen:expect([[
       {12:every}thing                    |
@@ -2524,14 +2721,14 @@ describe(":substitute", function()
     ]])
 
     feed([[<C-c>]])
-    wait()
+    poke_eventloop()
     feed([[:%s/some\(thing\)\@!/every/]])
     screen:expect([[
+      something                     |
+      everything                    |
       {12:every}one                      |
       {15:~                             }|
       {15:~                             }|
-      {15:~                             }|
-      {15:~                             }|
       {11:[No Name] [+]                 }|
       |3| {12:every}one                  |
       {15:~                             }|
@@ -2544,6 +2741,49 @@ describe(":substitute", function()
       :%s/some\(thing\)\@!/every/^   |
     ]])
   end)
+
+  it("doesn't prompt to swap cmd range", function()
+    screen = Screen.new(50, 8) -- wide to avoid hit-enter prompt
+    common_setup(screen, "split", default_text)
+    feed(':2,1s/tw/MO/g')
+
+    -- substitution preview should have been made, without prompting
+    screen:expect([[
+      {12:MO}o lines                                         |
+      {11:[No Name] [+]                                     }|
+      |2| {12:MO}o lines                                     |
+      {15:~                                                 }|
+      {15:~                                                 }|
+      {15:~                                                 }|
+      {10:[Preview]                                         }|
+      :2,1s/tw/MO/g^                                     |
+    ]])
+
+    -- but should be prompted on hitting enter
+    feed('<CR>')
+    screen:expect([[
+      {12:MO}o lines                                         |
+      {11:[No Name] [+]                                     }|
+      |2| {12:MO}o lines                                     |
+      {15:~                                                 }|
+      {15:~                                                 }|
+      {15:~                                                 }|
+      {10:[Preview]                                         }|
+      {13:Backwards range given, OK to swap (y/n)?}^          |
+    ]])
+
+    feed('y')
+    screen:expect([[
+      Inc substitution on                               |
+      ^MOo lines                                         |
+                                                        |
+      {15:~                                                 }|
+      {15:~                                                 }|
+      {15:~                                                 }|
+      {15:~                                                 }|
+      {13:Backwards range given, OK to swap (y/n)?}y         |
+    ]])
+  end)
 end)
 
 it(':substitute with inccommand during :terminal activity', function()
@@ -2565,7 +2805,7 @@ it(':substitute with inccommand during :terminal activity', function()
     feed('gg')
     feed(':%s/foo/ZZZ')
     sleep(20)  -- Allow some terminal activity.
-    helpers.wait()
+    helpers.poke_eventloop()
     screen:expect_unchanged()
   end)
 end)
@@ -2597,6 +2837,26 @@ it(':substitute with inccommand, timer-induced :redraw #9777', function()
   ]])
 end)
 
+it(":substitute doesn't crash with inccommand, if undo is empty #12932", function()
+  local screen = Screen.new(10,5)
+  clear()
+  command('set undolevels=-1')
+  common_setup(screen, 'split', 'test')
+  feed(':%s/test')
+  sleep(100)
+  feed('/')
+  sleep(100)
+  feed('f')
+  screen:expect([[
+  {12:f}           |
+  {15:~           }|
+  {15:~           }|
+  {15:~           }|
+  :%s/test/f^  |
+  ]])
+  assert_alive()
+end)
+
 it('long :%s/ with inccommand does not collapse cmdline', function()
   local screen = Screen.new(10,5)
   clear()
